-module(etcp_acceptor_root_sup).
-author("pouriya.jahanbakhsh@gmail.com").


-export([start_link/4
        ,fetch/1
        ,sleep/1
        ,wakeup/1
        ,fetch_mode/1
        ,start_acceptor_sup/2]).

-export([init/1
        ,terminate/2]).

-define(DEF_START_OPTS, []).
-define(DEF_ACCEPTOR_COUNT, 3).
-include("internal/etcp_guard.hrl").
-define(SLEEP, 'sleep').
-define(ACCEPT, 'accept').


-spec
start_link(module(), term(), etcp_types:start_options(), etcp_types:socket()) ->
    etcp_types:start_return().
start_link(Mod, InitArg, Opts, LSock) when erlang:is_map(Opts) ->
    director:start_link(?MODULE, {Mod, InitArg, Opts, LSock}, ?DEF_START_OPTS).


-spec
fetch(etcp_types:name()) ->
    [] | [{reference(), pid()}].
%% @doc
%%      returns all acceptors.
%% @end
fetch(AccRootSup) when ?is_proc_ref(AccRootSup) ->
    {ok, Pids} = director:get_pids(AccRootSup),
    Fold =
        fun
            ({ok, AccInfo}, Acc) ->
                [AccInfo|Acc];
            (_, Acc) ->
                Acc
        end,
    lists:foldl(Fold, [], [etcp_acceptor_sup:fetch(AccSupPid) || {_, AccSupPid} <- Pids]).


-spec
sleep(etcp_types:name()) ->
    'ok'.
sleep(AccRootSup) when ?is_proc_ref(AccRootSup) ->
    {ok, AccSups} = director:get_pids(AccRootSup),
    ForEach =
        fun({_, AccPid}) ->
            etcp_acceptor_sup:sleep(AccPid)
        end,
    lists:foreach(ForEach, AccSups).


-spec
wakeup(etcp_types:name()) ->
    'ok'.
wakeup(AccRootSup) when ?is_proc_ref(AccRootSup) ->
    {ok, AccSups} = director:get_pids(AccRootSup),
    ForEach =
        fun({_, AccPid}) ->
            etcp_acceptor_sup:wakeup(AccPid)
        end,
    lists:foreach(ForEach, AccSups).


-spec
fetch_mode(etcp_types:name()) ->
    etcp_types:acceptor_mode() | [etcp_types:acceptor_mode()].
fetch_mode(AccRootSup) when ?is_proc_ref(AccRootSup) ->
    {ok, AccSups} = director:get_pids(AccRootSup),
    Modes = [Mode
            || Mode <- [etcp_acceptor_sup:fetch_mode(AccSupPid) || {_, AccSupPid} <- AccSups]
            ,Mode == ?ACCEPT orelse Mode == ?SLEEP],
    case Modes of
        [Mode|_] ->
            All =
                fun(Mode_) ->
                    if
                        Mode == Mode_ ->
                            true;
                        true ->
                            false
                    end
                end,
            case lists:all(All, Modes) of
                true ->
                    Mode;
                false ->
                    Modes
            end;
        _ ->
            Modes
    end.


-spec
start_acceptor_sup(etcp_types:name(), etcp_types:name()) ->
    'ok' | etcp_types:error().
start_acceptor_sup(AccRootSup, ConSup) when ?is_proc_ref(AccRootSup) andalso ?is_proc_ref(ConSup) ->
    case director:start_child(AccRootSup, #{start => {etcp_acceptor_sup, start_link, [ConSup]}
                                           ,id => erlang:make_ref()
                                           ,append => true}) of
        {ok, _} ->
            ok;
        {error, _}=Err ->
            Err
    end.


init({Mod, InitArg, Opts, LSock}) ->
    DefChild = #{start => {etcp_acceptor_sup, start_link, [Mod, InitArg, Opts, LSock]}
                ,plan => fun(_, _, _, St) -> {stop, St} end
                ,type => s
                ,log_validator => fun child_log_validator/4},
    Opts2 = [{log_validator, fun sup_log_validator/4}],
    {ok, undefined, [], DefChild, Opts2}.


terminate(_Rsn, _St) ->
    ok.


child_log_validator(_Name, info, start, _State) ->
    none;
child_log_validator(_Name, error, normal, _State) ->
    none;
child_log_validator(_Name, error, _, _State) ->
    long.


sup_log_validator(_Name, error, _, _State) ->
    none;
sup_log_validator(_Name, warning, _, _State) ->
    long.