%%% ------------------------------------------------------------------------------------------------
%%% ETCP is available for use under the following license, commonly known as the 3-clause (or
%%% "modified") BSD license:
%%%
%%% Copyright (c) 2017-2018, Pouriya Jahanbakhsh
%%% (pouriya.jahanbakhsh@gmail.com)
%%% All rights reserved.
%%%
%%% Redistribution and use in source and binary forms, with or without modification, are permitted
%%% provided that the following conditions are met:
%%%
%%% 1. Redistributions of source code must retain the above copyright notice, this list of
%%%    conditions and the following disclaimer.
%%%
%%% 2. Redistributions in binary form must reproduce the above copyright notice, this list of
%%%    conditions and the following disclaimer in the documentation and/or other materials provided
%%%    with the distribution.
%%%
%%% 3. Neither the name of the copyright holder nor the names of its contributors may be used to
%%%    endorse or promote products derived from this software without specific prior written
%%%    permission.
%%%
%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
%%% IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
%%% FITNESS FOR A  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
%%% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
%%% THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
%%% OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
%%% POSSIBILITY OF SUCH DAMAGE.
%%% ------------------------------------------------------------------------------------------------
%% @author  Pouriya Jahanbakhsh <pouriya.jahanbakhsh@gmail.com>
%% @version 17.9.10
%% -------------------------------------------------------------------------------------------------
-module(etcp_metadata).
-author("pouriya.jahanbakhsh@gmail.com").
%% -------------------------------------------------------------------------------------------------
%% Exports:

%% API:
-export([wrap/9
        ,unwrap/1

        ,fetch_socket/1
        ,fetch_timeout/1
        ,fetch_srtimeout/1
        ,fetch_length/1
        ,fetch_transporter/1
        ,fetch_options/1
        ,fetch_last_message/1
        ,fetch_last_callback/1
        ,fetch_address/1
        ,fetch_host/1
        ,fetch_port/1]).

%% -------------------------------------------------------------------------------------------------
%% Records & Macros & Includes:

-include("internal/etcp_metadata.hrl").
-define(ETCP_MD, etcp_metadata).

%% -------------------------------------------------------------------------------------------------
%% API:

-spec
wrap(etcp_types:socket()
    ,timeout()
    ,timeout()
    , etcp_types:length()
    ,module()
    , etcp_types:start_options()
    ,any()
    ,atom()
    , etcp_types:address()) ->
    etcp_types:metadata().
wrap(Sock, Timeout, SRTimeout, Len, TrMod, Opts, LMsg, LCB, Addr) ->
    #?ETCP_MD{socket = Sock
         ,timeout = Timeout
         ,length = Len
         ,srtimeout = SRTimeout
         ,transporter = TrMod
         ,options = Opts
         ,last_message = LMsg
         ,last_callback = LCB
         ,address = Addr}.


-spec
unwrap(etcp_types:metadata()) ->
    {etcp_types:socket()
    ,timeout()
    ,timeout()
    , etcp_types:length()
    ,module()
    , etcp_types:start_options()
    ,any()
    ,atom()
    , etcp_types:address()}.
unwrap(#?ETCP_MD{socket = Sock
            ,timeout = Timeout
            ,length = Len
            ,srtimeout = SRTimeout
            ,transporter = TrMod
            ,options = Opts
            ,last_message = LMsg
            ,last_callback = LCB
            ,address = Addr}) ->
    {Sock, Timeout, Len, SRTimeout, TrMod, Opts, LMsg, LCB, Addr}.


-spec
fetch_socket(etcp_types:metadata()) ->
    etcp_types:socket().
fetch_socket(#?ETCP_MD{socket = Sock}) ->
    Sock.


-spec
fetch_timeout(etcp_types:metadata()) ->
    timeout().
fetch_timeout(#?ETCP_MD{timeout = Timeout}) ->
    Timeout.


-spec
fetch_srtimeout(etcp_types:metadata()) ->
    timeout().
fetch_srtimeout(#?ETCP_MD{srtimeout = SRTimeout}) ->
    SRTimeout.


-spec
fetch_length(etcp_types:metadata()) ->
    etcp_types:length().
fetch_length(#?ETCP_MD{length = Len}) ->
    Len.


-spec
fetch_transporter(etcp_types:metadata()) ->
    module().
fetch_transporter(#?ETCP_MD{transporter = TrMod}) ->
    TrMod.


-spec
fetch_options(etcp_types:metadata()) ->
    etcp_types:start_options().
fetch_options(#?ETCP_MD{options = Opts}) ->
    Opts.


-spec
fetch_last_message(etcp_types:metadata()) ->
    any().
fetch_last_message(#?ETCP_MD{last_message = LMsg}) ->
    LMsg.


-spec
fetch_last_callback(etcp_types:metadata()) ->
    atom().
fetch_last_callback(#?ETCP_MD{last_callback = LCB}) ->
    LCB.


-spec
fetch_address(etcp_types:metadata()) ->
    etcp_types:address().
fetch_address(#?ETCP_MD{address = Addr}) ->
    Addr.


-spec
fetch_host(etcp_types:metadata()) ->
    etcp_types:host() | 'undefined'.
fetch_host(#?ETCP_MD{address = {Host, _}}) ->
    Host.


-spec
fetch_port(etcp_types:metadata()) ->
    etcp_types:port_number() | 'undefined'.
fetch_port(#?ETCP_MD{address = {_, Port}}) ->
    Port.


